# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import AcquireAction
from .Contents import Contents
from ...core.model import ConsumeAction
from .options.QuestModelOptions import QuestModelOptions


class QuestModel:
    name: str
    contents: List[Contents]
    metadata: Optional[str] = None
    challenge_period_event_id: Optional[str] = None
    first_complete_acquire_actions: Optional[List[AcquireAction]] = None
    consume_actions: Optional[List[ConsumeAction]] = None
    failed_acquire_actions: Optional[List[AcquireAction]] = None
    premise_quest_names: Optional[List[str]] = None

    def __init__(
        self,
        name: str,
        contents: List[Contents],
        options: Optional[QuestModelOptions] = QuestModelOptions(),
    ):
        self.name = name
        self.contents = contents
        self.metadata = options.metadata if options.metadata else None
        self.challenge_period_event_id = options.challenge_period_event_id if options.challenge_period_event_id else None
        self.first_complete_acquire_actions = options.first_complete_acquire_actions if options.first_complete_acquire_actions else None
        self.consume_actions = options.consume_actions if options.consume_actions else None
        self.failed_acquire_actions = options.failed_acquire_actions if options.failed_acquire_actions else None
        self.premise_quest_names = options.premise_quest_names if options.premise_quest_names else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.contents is not None:
            properties["contents"] = [
                v.properties(
                )
                for v in self.contents
            ]
        if self.challenge_period_event_id is not None:
            properties["challengePeriodEventId"] = self.challenge_period_event_id
        if self.first_complete_acquire_actions is not None:
            properties["firstCompleteAcquireActions"] = [
                v.properties(
                )
                for v in self.first_complete_acquire_actions
            ]
        if self.consume_actions is not None:
            properties["consumeActions"] = [
                v.properties(
                )
                for v in self.consume_actions
            ]
        if self.failed_acquire_actions is not None:
            properties["failedAcquireActions"] = [
                v.properties(
                )
                for v in self.failed_acquire_actions
            ]
        if self.premise_quest_names is not None:
            properties["premiseQuestNames"] = self.premise_quest_names

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.GitHubApiKeyOptions import GitHubApiKeyOptions


class GitHubApiKey:
    name: str
    api_key: str
    encryption_key_name: str
    description: Optional[str] = None
    revision: Optional[int] = None

    def __init__(
        self,
        name: str,
        api_key: str,
        encryption_key_name: str,
        options: Optional[GitHubApiKeyOptions] = GitHubApiKeyOptions(),
    ):
        self.name = name
        self.api_key = api_key
        self.encryption_key_name = encryption_key_name
        self.description = options.description if options.description else None
        self.revision = options.revision if options.revision else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.description is not None:
            properties["description"] = self.description
        if self.api_key is not None:
            properties["apiKey"] = self.api_key
        if self.encryption_key_name is not None:
            properties["encryptionKeyName"] = self.encryption_key_name

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .BonusModel import BonusModel
from .options.BonusModelOptions import BonusModelOptions
from .enum.BonusModelMode import BonusModelMode
from .enum.BonusModelRepeat import BonusModelRepeat
from .enum.BonusModelMissedReceiveRelief import BonusModelMissedReceiveRelief
from .options.BonusModelModeIsScheduleOptions import BonusModelModeIsScheduleOptions
from .options.BonusModelModeIsStreamingOptions import BonusModelModeIsStreamingOptions
from .options.BonusModelMissedReceiveReliefIsEnabledOptions import BonusModelMissedReceiveReliefIsEnabledOptions
from .options.BonusModelMissedReceiveReliefIsDisabledOptions import BonusModelMissedReceiveReliefIsDisabledOptions
from .Reward import Reward
from .options.RewardOptions import RewardOptions
from .CurrentMasterData import CurrentMasterData


from __future__ import annotations
from typing import List


class Func:
    pass


class GetAttr(Func):

    key: str

    def __init__(
            self,
            resource: 'gs2_cdk.CdkResource' = None,
            path: str = None,
            key: str = None
    ):
        if key is None:
            self.key = "{}.{}".format(
                resource.resource_name,
                path,
            )
        else:
            self.key = key

    def str(self) -> str:
        return "!GetAttr {}".format(
            self.key,
        )

    @staticmethod
    def region() -> GetAttr:
        return GetAttr(
            key="Gs2::Region",
        )

    @staticmethod
    def owner_id() -> GetAttr:
        return GetAttr(
            key="Gs2::OwnerId",
        )


class Join(Func):

    separator: str
    values: List[str]

    def __init__(
            self,
            separator: str,
            values: List[str],
    ):
        self.separator = separator
        self.values = values

    def str(self) -> str:
        return "!Join ['{}', [{}]]".format(
            self.separator,
            ', '.join([
                "'{}'".format(value)
                for value in self.values
            ]),
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
#
# deny overwrite
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr

from ..ref.StateMachineMasterRef import StateMachineMasterRef

from .options.StateMachineMasterOptions import StateMachineMasterOptions


class StateMachineMaster(CdkResource):
    stack: Stack
    namespace_name: str
    main_state_machine_name: str
    payload: str

    def __init__(
        self,
        stack: Stack,
        namespace_name: str,
        main_state_machine_name: str,
        payload: str,
        options: Optional[StateMachineMasterOptions] = StateMachineMasterOptions(),
    ):
        super().__init__(
            "StateMachine_StateMachineMaster_" + namespace_name
        )

        self.stack = stack
        self.namespace_name = namespace_name
        self.main_state_machine_name = main_state_machine_name
        self.payload = payload
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "version"

    def resource_type(
        self,
    ) -> str:
        return "GS2::StateMachine::StateMachineMaster"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.namespace_name is not None:
            properties["NamespaceName"] = self.namespace_name
        if self.main_state_machine_name is not None:
            properties["MainStateMachineName"] = self.main_state_machine_name
        if self.payload is not None:
            properties["Payload"] = self.payload

        return properties

    def ref(
        self,
        version: int,
    ) -> StateMachineMasterRef:
        return StateMachineMasterRef(
            self.namespace_name,
            version,
        )

    def get_attr_state_machine_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            None,
            None,
            "Item.StateMachineId",
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from ..stamp_sheet.RecoverStaminaByUserId import RecoverStaminaByUserId
from ..stamp_sheet.RaiseMaxValueByUserId import RaiseMaxValueByUserId
from ..stamp_sheet.SetMaxValueByUserId import SetMaxValueByUserId
from ..stamp_sheet.SetRecoverIntervalByUserId import SetRecoverIntervalByUserId
from ..stamp_sheet.SetRecoverValueByUserId import SetRecoverValueByUserId
from ..stamp_sheet.DecreaseMaxValueByUserId import DecreaseMaxValueByUserId
from ..stamp_sheet.ConsumeStaminaByUserId import ConsumeStaminaByUserId


class StaminaModelRef:
    namespace_name: str
    stamina_name: str

    def __init__(
        self,
        namespace_name: str,
        stamina_name: str,
    ):
        self.namespace_name = namespace_name
        self.stamina_name = stamina_name

    def recover_stamina(
        self,
        recover_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> RecoverStaminaByUserId:
        return RecoverStaminaByUserId(
            self.namespace_name,
            self.stamina_name,
            recover_value,
            user_id,
        )

    def raise_max_value(
        self,
        raise_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> RaiseMaxValueByUserId:
        return RaiseMaxValueByUserId(
            self.namespace_name,
            self.stamina_name,
            raise_value,
            user_id,
        )

    def set_max_value(
        self,
        max_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> SetMaxValueByUserId:
        return SetMaxValueByUserId(
            self.namespace_name,
            self.stamina_name,
            max_value,
            user_id,
        )

    def set_recover_interval(
        self,
        recover_interval_minutes: int,
        user_id: Optional[str] = "#{userId}",
    ) -> SetRecoverIntervalByUserId:
        return SetRecoverIntervalByUserId(
            self.namespace_name,
            self.stamina_name,
            recover_interval_minutes,
            user_id,
        )

    def set_recover_value(
        self,
        recover_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> SetRecoverValueByUserId:
        return SetRecoverValueByUserId(
            self.namespace_name,
            self.stamina_name,
            recover_value,
            user_id,
        )

    def decrease_max_value(
        self,
        decrease_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> DecreaseMaxValueByUserId:
        return DecreaseMaxValueByUserId(
            self.namespace_name,
            self.stamina_name,
            decrease_value,
            user_id,
        )

    def consume_stamina(
        self,
        consume_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> ConsumeStaminaByUserId:
        return ConsumeStaminaByUserId(
            self.namespace_name,
            self.stamina_name,
            consume_value,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "stamina",
                self.namespace_name,
                "model",
                self.stamina_name,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .Attribute import Attribute
from .Player import Player
from .options.CapacityOfRoleOptions import CapacityOfRoleOptions


class CapacityOfRole:
    role_name: str
    capacity: int
    role_aliases: Optional[List[str]] = None
    participants: Optional[List[Player]] = None

    def __init__(
        self,
        role_name: str,
        capacity: int,
        options: Optional[CapacityOfRoleOptions] = CapacityOfRoleOptions(),
    ):
        self.role_name = role_name
        self.capacity = capacity
        self.role_aliases = options.role_aliases if options.role_aliases else None
        self.participants = options.participants if options.participants else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.role_name is not None:
            properties["roleName"] = self.role_name
        if self.role_aliases is not None:
            properties["roleAliases"] = self.role_aliases
        if self.capacity is not None:
            properties["capacity"] = self.capacity
        if self.participants is not None:
            properties["participants"] = [
                v.properties(
                )
                for v in self.participants
            ]

        return properties
